/*
lkmonitor (Linux Kernel Monitor)

Application for monitoring and tuning a Linux kernel.

Copyright (C) 2005-2006  Fernando Apesteguía

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
*/

#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <gnome.h>
#include <string.h>
#include <stdio.h>
#include "lininfo.h"
#include <sys/types.h>
#include <dirent.h>

#include <glib.h>
#include <pthread.h>
#include <gtk/gtk.h>

/*! Read integer values from file.
* \param parm Number of parameters
* \param path File to read
* \param ... variable list of parameters to read. Should be (int*)
* \return 0 on success, -1 on error
*/
int read_values_from_file(int parm,char *path, ...)
{
	g_assert(parm!=0 && path!=NULL);
	
	int i;
	int *val;
	va_list list;
	gchar *contents;
	char *p;
	
	if(g_file_get_contents((gchar *)path,&contents,NULL,NULL)==FALSE){
		va_start(list,path);
		for(i=0; i<parm;i++){
			val=va_arg(list, int *);
			*val=-1;
		}
		va_end(list);
		g_warning("Failed to open %s file",path);
		return -1;
	}
	
	p=contents;
	
	va_start(list,path);
	for(i=0; i<parm;i++){
		sscanf(p,"%d",va_arg(list, int *));
		p=strchr(p,'\t')+1;
	}
	va_end(list);
	
	g_free(contents);
	return(0);
}

/*! Write integer values from file.
* \param parm Number of parameters
* \param path File to read
* \param ... variable list of parameters to write. Should be int
* \return 0 on success, -1 on error
*/
int write_values_to_file(int parm, char *path,...)
{
	g_assert(parm!=0 && path!=NULL);
	
	int i;
	va_list list;
	FILE *target;
	gchar num_trans[15]; //Large enough
	GString *ms;
	
	ms=g_string_new("");
	
	if((target=fopen(path,"w"))==NULL){
		g_warning("Can't write to %s\n",path);
		return(-1);
	}
	
	/* File is ready to be written */

	va_start(list,path);
	for(i=0;i<parm;i++){
		sprintf(num_trans,"%d",va_arg(list,int));
		g_string_append(ms,num_trans);
		g_string_append(ms," ");
	}
	va_end(list);

	/* Write */
	fprintf(target,"%s",ms->str);
	fclose(target);
	return(0);
}

/*! Write a string to file.
* \param str String to be written
* \param path File to write
* \return 0 on success, -1 on error
*/
int write_string_to_file(char *str,char *path)
{
  g_assert(str!=NULL && path!=NULL);
	
  FILE *f;
  
  if((f=fopen(path,"w"))==NULL){
	  g_warning("Couldn't open file %s",path);
	  return -1;
  }
  
  fprintf(f,"%s",str);
  printf("%s",str);
  fclose(f);
  
  return(0);
}


/*! Deletes blanks from the begining of string
* \param str String to delete blanks from
* \return Trimed String
*/
char *
trim (char *str)
{
  g_assert(str!=NULL);
  return (str + strspn (str, " "));
}				///End trim


/*! Deletes blanks from the end of string.
* \param str String to delete blanks from
* \return Trimed String
*/
char *
rtrim(char* str)
{
	g_assert(str!=NULL);
	
	char* p;
	if((p=rindex(str,' '))==NULL)
		return(str);
	while(p>str){
		if(*p==' '){
			*p='\0';
			p--;
		}
		else return(str);
	}
	return(str);//For gcc warning	
}

/*! Delete initial lines form string.
* \param contents String that contents text
* \param num_lines Number of lines to leave out
* \return 0 on success
*/
char *
skip_lines (char* contents, int num_lines)
{
  g_assert(contents!=NULL && num_lines>=0);
	
  int i, c;

  for (i = 0; i < num_lines; c++)
    if (*contents=='\n'){
      i++;
	}else{
       contents++;
	}

  return (contents+1);
}

/*! Main parse function.
* This function actually parses the contents of procfs files and tokenize
* elements to offer clean data.
* \param pattern sscanf pattern format to make parsing
* \param member Field struct to fill with parsed data
* \param contents Contents of the file to be analyzed
*/
void
find_data_for (char* pattern,char* member, const char* contents)
{	

	g_assert(pattern!=NULL && member!=NULL && contents!=NULL);
	
	char p[45];
	char *line;
	
	sscanf(pattern,"%[^\t:]",p);
	//p has the name of the attribute (model, model name...)
	strcpy(p,rtrim(p));
	if((line=strstr(contents,p))==NULL){
		strcpy(member,_("Unknown"));
		return;
	}//Position in the correct line

	sscanf(line, pattern,member);
	
}				///End find_data_for

/*! Parse battery information.
* \param str String to search in contents
* \param contents String that contents text
* \return 0 on success
*/
char *parse_batt(char *str,char *contents)
{
	g_assert(str!=NULL && contents!=NULL);
	
	char data[10];
	char *p;
	
	p=strstr(contents,str);
	if(p==NULL){
		return(_("Unknown"));
	}
	p=strstr(p,":")+1;
	sscanf(p,"%s",data);
	p=malloc(sizeof(data));
	strcpy(p,data);
	return p;
}
